@@ -1,7 +1,7 @@
 // synthv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@
 
 #include "synthv1_fx.h"
 
+#include "synthv1_reverb.h"
+
 
 #ifdef CONFIG_DEBUG_0
 #include <stdio.h>
@@ -47,7 +49,7 @@ const uint16_t MAX_VOICES = 32;			// polyphony
 const uint8_t  MAX_NOTES  = 128;
 
 const float MIN_ENV_MSECS = 2.0f;		// min 2msec per stage
-const float MAX_ENV_MSECS = 5000.0f;	// max 5 sec per stage
+const float MAX_ENV_MSECS = 5000.0f;	// max 5 sec per stage (default)
 
 const float DETUNE_SCALE  = 0.5f;
 const float PHASE_SCALE   = 0.5f;
@@ -306,14 +308,17 @@ struct synthv1_dco
 {
 	float *shape1;
 	float *width1;
+	float *bandl1;
 	float *shape2;
 	float *width2;
+	float *bandl2;
 	float *balance;
 	float *detune;
 	float *phase;
 	float *octave;
 	float *tuning;
 	float *glide;
+	float *envtime, envtime0;
 };
 
 
@@ -431,6 +436,18 @@ struct synthv1_del
 };
 
 
+// reverb
+
+struct synthv1_rev
+{
+	float *wet;
+	float *room;
+	float *damp;
+	float *feedb;
+	float *width;
+};
+
+
 // dynamic(compressor/limiter)
 
 struct synthv1_dyn
@@ -751,7 +768,7 @@ public:
 	synthv1_wave dco1_wave1, dco1_wave2;
 	synthv1_wave dco2_wave1, dco2_wave2;
 
-	synthv1_wave lfo1_wave,  lfo2_wave;
+	synthv1_wave_lf lfo1_wave, lfo2_wave;
 
 	float dco1_last1;
 	float dco1_last2;
@@ -760,6 +777,10 @@ public:
 
 protected:
 
+	void updateEnvTimes();
+	void updateEnvTimes_1();
+	void updateEnvTimes_2();
+
 	void allControllersOff();
 	void allControllersOff_1();
 	void allControllersOff_2();
@@ -805,6 +826,7 @@ private:
 	synthv1_fla m_fla;
 	synthv1_pha m_pha;
 	synthv1_del m_del;
+	synthv1_rev m_rev;
 	synthv1_dyn m_dyn;
 
 	synthv1_voice **m_voices;
@@ -825,6 +847,8 @@ private:
 	synthv1_fx_phaser  *m_phaser;
 	synthv1_fx_delay   *m_delay;
 	synthv1_fx_comp    *m_comp;
+
+	synthv1_reverb m_reverb;
 };
 
 
@@ -858,6 +882,9 @@ synthv1_voice::synthv1_voice ( synthv1_impl *pImpl ) :
 
 synthv1_impl::synthv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
 {
+	// max env. stage length (default)
+	m_dco1.envtime0 = m_dco2.envtime0 = 0.0001f * MAX_ENV_MSECS;
+
 	// glide notes
 	dco1_last1 = 0.0f;
 	dco1_last2 = 0.0f;
@@ -971,29 +998,7 @@ void synthv1_impl::setSampleRate ( uint32_t iSampleRate )
 	lfo1_wave.setSampleRate(m_iSampleRate);
 	lfo2_wave.setSampleRate(m_iSampleRate);
 
-	// update envelope range times in frames
-	const float srate_ms = 0.001f * float(m_iSampleRate);
-
-	const uint32_t min_frames = uint32_t(srate_ms * MIN_ENV_MSECS);
-	const uint32_t max_frames = uint32_t(srate_ms * MAX_ENV_MSECS);
-
-	m_dcf1.env.min_frames = min_frames;
-	m_dcf1.env.max_frames = max_frames;
-
-	m_dcf2.env.min_frames = min_frames;
-	m_dcf2.env.max_frames = max_frames;
-
-	m_lfo1.env.min_frames = min_frames;
-	m_lfo1.env.max_frames = max_frames;
-
-	m_lfo2.env.min_frames = min_frames;
-	m_lfo2.env.max_frames = max_frames;
-
-	m_dca1.env.min_frames = min_frames;
-	m_dca1.env.max_frames = max_frames;
-
-	m_dca2.env.min_frames = min_frames;
-	m_dca2.env.max_frames = max_frames;
+	updateEnvTimes();
 }
 
 
@@ -1003,6 +1008,59 @@ uint32_t synthv1_impl::sampleRate (void) const
 }
 
 
+void synthv1_impl::updateEnvTimes_1 (void)
+{
+	// update envelope range times in frames
+	const float srate_ms = 0.001f * float(m_iSampleRate);
+
+	float envtime_msecs = 10000.0f * m_dco1.envtime0;
+	if (envtime_msecs < MIN_ENV_MSECS)
+		envtime_msecs = MIN_ENV_MSECS + 1.0f;
+
+	const uint32_t min_frames = uint32_t(srate_ms * MIN_ENV_MSECS);
+	const uint32_t max_frames = uint32_t(srate_ms * envtime_msecs);
+
+	m_dcf1.env.min_frames = min_frames;
+	m_dcf1.env.max_frames = max_frames;
+
+	m_lfo1.env.min_frames = min_frames;
+	m_lfo1.env.max_frames = max_frames;
+
+	m_dca1.env.min_frames = min_frames;
+	m_dca1.env.max_frames = max_frames;
+}
+
+
+void synthv1_impl::updateEnvTimes_2 (void)
+{
+	// update envelope range times in frames
+	const float srate_ms = 0.001f * float(m_iSampleRate);
+
+	float envtime_msecs = 10000.0f * m_dco2.envtime0;
+	if (envtime_msecs < MIN_ENV_MSECS)
+		envtime_msecs = MIN_ENV_MSECS + 1.0f;
+
+	const uint32_t min_frames = uint32_t(srate_ms * MIN_ENV_MSECS);
+	const uint32_t max_frames = uint32_t(srate_ms * envtime_msecs);
+
+	m_dcf2.env.min_frames = min_frames;
+	m_dcf2.env.max_frames = max_frames;
+
+	m_lfo2.env.min_frames = min_frames;
+	m_lfo2.env.max_frames = max_frames;
+
+	m_dca2.env.min_frames = min_frames;
+	m_dca2.env.max_frames = max_frames;
+}
+
+
+void synthv1_impl::updateEnvTimes (void)
+{
+	updateEnvTimes_1();
+	updateEnvTimes_2();
+}
+
+
 void synthv1_impl::setParamPort ( synthv1::ParamIndex index, float *pfParam )
 {
 	static float s_fDummy = 0.0f;
@@ -1013,14 +1071,17 @@ void synthv1_impl::setParamPort ( synthv1::ParamIndex index, float *pfParam )
 	switch (index) {
 	case synthv1::DCO1_SHAPE1:    m_dco1.shape1      = pfParam; break;
 	case synthv1::DCO1_WIDTH1:    m_dco1.width1      = pfParam; break;
+	case synthv1::DCO1_BANDL1:    m_dco1.bandl1      = pfParam; break;
 	case synthv1::DCO1_SHAPE2:    m_dco1.shape2      = pfParam; break;
 	case synthv1::DCO1_WIDTH2:    m_dco1.width2      = pfParam; break;
+	case synthv1::DCO1_BANDL2:    m_dco1.bandl2      = pfParam; break;
 	case synthv1::DCO1_BALANCE:   m_dco1.balance     = pfParam; break;
 	case synthv1::DCO1_DETUNE:    m_dco1.detune      = pfParam; break;
 	case synthv1::DCO1_PHASE:     m_dco1.phase       = pfParam; break;
 	case synthv1::DCO1_OCTAVE:    m_dco1.octave      = pfParam; break;
 	case synthv1::DCO1_TUNING:    m_dco1.tuning      = pfParam; break;
 	case synthv1::DCO1_GLIDE:     m_dco1.glide       = pfParam; break;
+	case synthv1::DCO1_ENVTIME:   m_dco1.envtime     = pfParam; break;
 	case synthv1::DCF1_CUTOFF:    m_dcf1.cutoff      = pfParam; break;
 	case synthv1::DCF1_RESO:      m_dcf1.reso        = pfParam; break;
 	case synthv1::DCF1_TYPE:      m_dcf1.type        = pfParam; break;
@@ -1059,14 +1120,17 @@ void synthv1_impl::setParamPort ( synthv1::ParamIndex index, float *pfParam )
 	case synthv1::DEF1_MONO:      m_def1.mono        = pfParam; break;
 	case synthv1::DCO2_SHAPE1:    m_dco2.shape1      = pfParam; break;
 	case synthv1::DCO2_WIDTH1:    m_dco2.width1      = pfParam; break;
+	case synthv1::DCO2_BANDL1:    m_dco2.bandl1      = pfParam; break;
 	case synthv1::DCO2_SHAPE2:    m_dco2.shape2      = pfParam; break;
 	case synthv1::DCO2_WIDTH2:    m_dco2.width2      = pfParam; break;
+	case synthv1::DCO2_BANDL2:    m_dco2.bandl2      = pfParam; break;
 	case synthv1::DCO2_BALANCE:   m_dco2.balance     = pfParam; break;
 	case synthv1::DCO2_DETUNE:    m_dco2.detune      = pfParam; break;
 	case synthv1::DCO2_PHASE:     m_dco2.phase       = pfParam; break;
 	case synthv1::DCO2_OCTAVE:    m_dco2.octave      = pfParam; break;
 	case synthv1::DCO2_TUNING:    m_dco2.tuning      = pfParam; break;
 	case synthv1::DCO2_GLIDE:     m_dco2.glide       = pfParam; break;
+	case synthv1::DCO2_ENVTIME:   m_dco2.envtime     = pfParam; break;
 	case synthv1::DCF2_CUTOFF:    m_dcf2.cutoff      = pfParam; break;
 	case synthv1::DCF2_RESO:      m_dcf2.reso        = pfParam; break;
 	case synthv1::DCF2_TYPE:      m_dcf2.type        = pfParam; break;
@@ -1123,6 +1187,11 @@ void synthv1_impl::setParamPort ( synthv1::ParamIndex index, float *pfParam )
 	case synthv1::DEL1_BPM:       m_del.bpm          = pfParam; break;
 	case synthv1::DEL1_BPMSYNC:   m_del.bpmsync      = pfParam; break;
 	case synthv1::DEL1_BPMHOST:   m_del.bpmhost      = pfParam; break;
+	case synthv1::REV1_WET:       m_rev.wet          = pfParam; break;
+	case synthv1::REV1_ROOM:      m_rev.room         = pfParam; break;
+	case synthv1::REV1_DAMP:      m_rev.damp         = pfParam; break;
+	case synthv1::REV1_FEEDB:     m_rev.feedb        = pfParam; break;
+	case synthv1::REV1_WIDTH:     m_rev.width        = pfParam; break;
 	case synthv1::DYN1_COMPRESS:  m_dyn.compress     = pfParam; break;
 	case synthv1::DYN1_LIMITER:   m_dyn.limiter      = pfParam; break;
 	default: break;
@@ -1137,14 +1206,17 @@ float *synthv1_impl::paramPort ( synthv1::ParamIndex index )
 	switch (index) {
 	case synthv1::DCO1_SHAPE1:    pfParam = m_dco1.shape1;      break;
 	case synthv1::DCO1_WIDTH1:    pfParam = m_dco1.width1;      break;
+	case synthv1::DCO1_BANDL1:    pfParam = m_dco1.bandl1;      break;
 	case synthv1::DCO1_SHAPE2:    pfParam = m_dco1.shape2;      break;
 	case synthv1::DCO1_WIDTH2:    pfParam = m_dco1.width2;      break;
+	case synthv1::DCO1_BANDL2:    pfParam = m_dco1.bandl2;      break;
 	case synthv1::DCO1_BALANCE:   pfParam = m_dco1.balance;     break;
 	case synthv1::DCO1_DETUNE:    pfParam = m_dco1.detune;      break;
 	case synthv1::DCO1_PHASE:     pfParam = m_dco1.phase;       break;
 	case synthv1::DCO1_OCTAVE:    pfParam = m_dco1.octave;      break;
 	case synthv1::DCO1_TUNING:    pfParam = m_dco1.tuning;      break;
 	case synthv1::DCO1_GLIDE:     pfParam = m_dco1.glide;       break;
+	case synthv1::DCO1_ENVTIME:   pfParam = m_dco1.envtime;     break;
 	case synthv1::DCF1_CUTOFF:    pfParam = m_dcf1.cutoff;      break;
 	case synthv1::DCF1_RESO:      pfParam = m_dcf1.reso;        break;
 	case synthv1::DCF1_TYPE:      pfParam = m_dcf1.type;        break;
@@ -1183,14 +1255,17 @@ float *synthv1_impl::paramPort ( synthv1::ParamIndex index )
 	case synthv1::DEF1_MONO:      pfParam = m_def1.mono;        break;
 	case synthv1::DCO2_SHAPE1:    pfParam = m_dco2.shape1;      break;
 	case synthv1::DCO2_WIDTH1:    pfParam = m_dco2.width1;      break;
+	case synthv1::DCO2_BANDL1:    pfParam = m_dco2.bandl1;      break;
 	case synthv1::DCO2_SHAPE2:    pfParam = m_dco2.shape2;      break;
 	case synthv1::DCO2_WIDTH2:    pfParam = m_dco2.width2;      break;
+	case synthv1::DCO2_BANDL2:    pfParam = m_dco2.bandl2;      break;
 	case synthv1::DCO2_BALANCE:   pfParam = m_dco2.balance;     break;
 	case synthv1::DCO2_DETUNE:    pfParam = m_dco2.detune;      break;
 	case synthv1::DCO2_PHASE:     pfParam = m_dco2.phase;       break;
 	case synthv1::DCO2_OCTAVE:    pfParam = m_dco2.octave;      break;
 	case synthv1::DCO2_TUNING:    pfParam = m_dco2.tuning;      break;
 	case synthv1::DCO2_GLIDE:     pfParam = m_dco2.glide;       break;
+	case synthv1::DCO2_ENVTIME:   pfParam = m_dco2.envtime;     break;
 	case synthv1::DCF2_CUTOFF:    pfParam = m_dcf2.cutoff;      break;
 	case synthv1::DCF2_RESO:      pfParam = m_dcf2.reso;        break;
 	case synthv1::DCF2_TYPE:      pfParam = m_dcf2.type;        break;
@@ -1247,6 +1322,11 @@ float *synthv1_impl::paramPort ( synthv1::ParamIndex index )
 	case synthv1::DEL1_BPM:       pfParam = m_del.bpm;          break;
 	case synthv1::DEL1_BPMSYNC:   pfParam = m_del.bpmsync;      break;
 	case synthv1::DEL1_BPMHOST:   pfParam = m_del.bpmhost;      break;
+	case synthv1::REV1_WET:       pfParam = m_rev.wet;          break;
+	case synthv1::REV1_ROOM:      pfParam = m_rev.room;         break;
+	case synthv1::REV1_DAMP:      pfParam = m_rev.damp;         break;
+	case synthv1::REV1_FEEDB:     pfParam = m_rev.feedb;        break;
+	case synthv1::REV1_WIDTH:     pfParam = m_rev.width;        break;
 	case synthv1::DYN1_COMPRESS:  pfParam = m_dyn.compress;     break;
 	case synthv1::DYN1_LIMITER:   pfParam = m_dyn.limiter;      break;
 	default: break;
@@ -1363,17 +1443,18 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 				// pressure/after-touch
 				pv->pre1 = 0.0f;
 				pv->dca1_pre.reset(m_def1.pressure, &m_ctl1.pressure, &pv->pre1);
-				// phases
-				pv->dco1_sample1 = pv->dco1_osc1.start();
-				pv->dco1_sample2 = pv->dco1_osc2.start(*m_dco1.phase * PHASE_SCALE);
 				// frequencies
-				const float freq1 = float(key)
+				const float note1 = float(key)
 					+ *m_dco1.octave * OCTAVE_SCALE
 					+ *m_dco1.tuning * TUNING_SCALE;
 				const float detune1
 					= *m_dco1.detune * DETUNE_SCALE;
-				pv->dco1_freq1 = synthv1_freq(freq1 - detune1);
-				pv->dco1_freq2 = synthv1_freq(freq1 + detune1);
+				pv->dco1_freq1 = synthv1_freq(note1 - detune1);
+				pv->dco1_freq2 = synthv1_freq(note1 + detune1);
+				// phases
+				const float phase1 = *m_dco1.phase * PHASE_SCALE;
+				pv->dco1_sample1 = pv->dco1_osc1.start(  0.0f, pv->dco1_freq1);
+				pv->dco1_sample2 = pv->dco1_osc2.start(phase1, pv->dco1_freq2);
 				// filters
 				const int type1 = int(*m_dcf1.type);
 				pv->dcf11.reset(synthv1_filter1::Type(type1));
@@ -1406,16 +1487,18 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 				// pressure/after-touch
 				pv->pre2 = 0.0f;
 				pv->dca2_pre.reset(m_def2.pressure, &m_ctl2.pressure, &pv->pre2);
-				// phases
-				pv->dco2_sample1 = pv->dco2_osc1.start();
-				pv->dco2_sample2 = pv->dco2_osc2.start(*m_dco2.phase * PHASE_SCALE);
-				const float freq2 = float(key)
+				// frequencies
+				const float note2 = float(key)
 					+ *m_dco2.octave * OCTAVE_SCALE
 					+ *m_dco2.tuning * TUNING_SCALE;
 				const float detune2
 					= *m_dco2.detune * DETUNE_SCALE;
-				pv->dco2_freq1 = synthv1_freq(freq2 - detune2);
-				pv->dco2_freq2 = synthv1_freq(freq2 + detune2);
+				pv->dco2_freq1 = synthv1_freq(note2 - detune2);
+				pv->dco2_freq2 = synthv1_freq(note2 + detune2);
+				// phases
+				const float phase2 = *m_dco2.phase * PHASE_SCALE;
+				pv->dco2_sample1 = pv->dco2_osc1.start(  0.0f, pv->dco2_freq1);
+				pv->dco2_sample2 = pv->dco2_osc2.start(phase2, pv->dco2_freq2);
 				// filters
 				const int type2 = int(*m_dcf2.type);
 				pv->dcf21.reset(synthv1_filter1::Type(type2));
@@ -1695,6 +1778,9 @@ void synthv1_impl::allSoundOff (void)
 		m_delay[k].reset();
 		m_comp[k].reset();
 	}
+
+	m_reverb.setSampleRate(m_iSampleRate);
+	m_reverb.reset();
 }
 
 
@@ -1731,6 +1817,9 @@ void synthv1_impl::reset (void)
 	if (m_comp == 0)
 		m_comp = new synthv1_fx_comp [m_iChannels];
 
+	// reverbs
+	m_reverb.reset();
+
 	allSoundOff();
 //	allControllersOff();
 	allNotesOff();
@@ -1754,25 +1843,42 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 	const float lfo1_rate = *m_lfo1.rate * *m_lfo1.rate;
 	const float lfo2_rate = *m_lfo2.rate * *m_lfo2.rate;
-	const float lfo1_freq = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
-	const float lfo2_freq = LFO_FREQ_MIN + lfo2_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+
+	const float lfo1_freq
+		= LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+	const float lfo2_freq
+		= LFO_FREQ_MIN + lfo2_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+
 	const float modwheel1 = m_ctl1.modwheel + PITCH_SCALE * *m_lfo1.pitch;
 	const float modwheel2 = m_ctl2.modwheel + PITCH_SCALE * *m_lfo2.pitch;
 
-	if (int(*m_dco1.shape1) != int(dco1_wave1.shape()) || *m_dco1.width1 != dco1_wave1.width())
-		dco1_wave1.reset(synthv1_wave::Shape(*m_dco1.shape1), *m_dco1.width1);
-	if (int(*m_dco1.shape2) != int(dco1_wave2.shape()) || *m_dco1.width2 != dco1_wave2.width())
-		dco1_wave2.reset(synthv1_wave::Shape(*m_dco1.shape2), *m_dco1.width2);
+	if (m_dco1.envtime0 != *m_dco1.envtime) {
+		m_dco1.envtime0  = *m_dco1.envtime;
+		updateEnvTimes_1();
+	}
+	if (m_dco2.envtime0 != *m_dco2.envtime) {
+		m_dco2.envtime0  = *m_dco2.envtime;
+		updateEnvTimes_2();
+	}
 
-	if (int(*m_dco2.shape1) != int(dco2_wave1.shape()) || *m_dco2.width1 != dco2_wave1.width())
-		dco2_wave1.reset(synthv1_wave::Shape(*m_dco2.shape1), *m_dco2.width1);
-	if (int(*m_dco2.shape2) != int(dco2_wave2.shape()) || *m_dco2.width2 != dco2_wave2.width())
-		dco2_wave2.reset(synthv1_wave::Shape(*m_dco2.shape2), *m_dco2.width2);
+	dco1_wave1.reset_test(
+		synthv1_wave::Shape(*m_dco1.shape1),
+		*m_dco1.width1, *m_dco1.bandl1 > 0.0f);
+	dco1_wave2.reset_test(
+		synthv1_wave::Shape(*m_dco1.shape2),
+		*m_dco1.width2, *m_dco1.bandl2 > 0.0f);
+
+	dco2_wave1.reset_test(
+		synthv1_wave::Shape(*m_dco2.shape1),
+		*m_dco2.width1, *m_dco2.bandl1 > 0.0f);
+	dco2_wave2.reset_test(
+		synthv1_wave::Shape(*m_dco2.shape2),
+		*m_dco2.width2, *m_dco2.bandl2 > 0.0f);
 	
-	if (int(*m_lfo1.shape) != int(lfo1_wave.shape()) || *m_lfo1.width != lfo1_wave.width())
-		lfo1_wave.reset(synthv1_wave::Shape(*m_lfo1.shape), *m_lfo1.width);
-	if (int(*m_lfo2.shape) != int(lfo2_wave.shape()) || *m_lfo2.width != lfo2_wave.width())
-		lfo2_wave.reset(synthv1_wave::Shape(*m_lfo2.shape), *m_lfo2.width);
+	lfo1_wave.reset_test(
+		synthv1_wave::Shape(*m_lfo1.shape), *m_lfo1.width);
+	lfo2_wave.reset_test(
+		synthv1_wave::Shape(*m_lfo2.shape), *m_lfo2.width);
 
 	// per voice
 
@@ -1966,16 +2072,15 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		m_del.bpm = del_bpmhost;
 	}
 
-	// effects
+	// chorus
+	if (m_iChannels > 1) {
+		m_chorus.process(outs[0], outs[1], nframes, *m_cho.wet,
+			*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
+	}
 
+	// effects
 	for (k = 0; k < m_iChannels; ++k) {
 		float *in = outs[k];
-		float *out = in;
-		// chorus
-		if (k > 0) {
-			m_chorus.process(outs[k - 1], outs[k], nframes, *m_cho.wet,
-				*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
-		}
 		// flanger
 		m_flanger[k].process(in, nframes, *m_fla.wet,
 			*m_fla.delay, *m_fla.feedb, *m_fla.daft * float(k));
@@ -1985,11 +2090,23 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		// delay
 		m_delay[k].process(in, nframes, *m_del.wet,
 			*m_del.delay, *m_del.feedb, *m_del.bpm);
+	}
+
+	// reverb
+	if (m_iChannels > 1) {
+		m_reverb.process(outs[0], outs[1], nframes, *m_rev.wet,
+			*m_rev.feedb, *m_rev.room, *m_rev.damp, *m_rev.width);
+	}
+
+	// dynamics
+	for (k = 0; k < m_iChannels; ++k) {
+		float *in = outs[k];
 		// compressor
 		if (int(*m_dyn.compress) > 0)
 			m_comp[k].process(in, nframes);
 		// limiter
 		if (int(*m_dyn.limiter) > 0) {
+			float *out = in;
 			for (uint32_t n = 0; n < nframes; ++n)
 				*out++ = synthv1_sigmoid(*in++);
 		}
