@@ -1,7 +1,7 @@
 // synthv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -35,14 +35,17 @@ struct {
 
 	{ "DCO1_SHAPE1",    1.0f },
 	{ "DCO1_WIDTH1",    1.0f },
+	{ "DCO1_BANDL1",    0.0f },
 	{ "DCO1_SHAPE2",    1.0f },
 	{ "DCO1_WIDTH2",    1.0f },
+	{ "DCO1_BANDL2",    0.0f },
 	{ "DCO1_BALANCE",   0.0f },
 	{ "DCO1_DETUNE",    0.1f },
 	{ "DCO1_PHASE",     0.0f },
 	{ "DCO1_OCTAVE",    0.0f },
 	{ "DCO1_TUNING",    0.0f },
 	{ "DCO1_GLIDE",     0.0f },
+	{ "DCO1_ENVTIME",   0.5f },
 	{ "DCF1_CUTOFF",    0.5f },
 	{ "DCF1_RESO",      0.0f },
 	{ "DCF1_TYPE",      0.0f },
@@ -83,14 +86,17 @@ struct {
 
 	{ "DCO2_SHAPE1",    1.0f },
 	{ "DCO2_WIDTH1",    1.0f },
+	{ "DCO2_BANDL1",    0.0f },
 	{ "DCO2_SHAPE2",    1.0f },
 	{ "DCO2_WIDTH2",    1.0f },
+	{ "DCO2_BANDL2",    0.0f },
 	{ "DCO2_BALANCE",   0.0f },
 	{ "DCO2_DETUNE",    0.1f },
 	{ "DCO2_PHASE",     0.0f },
 	{ "DCO2_OCTAVE",   -2.0f },
 	{ "DCO2_TUNING",    0.0f },
 	{ "DCO2_GLIDE",     0.0f },
+	{ "DCO2_ENVTIME",   0.5f },
 	{ "DCF2_CUTOFF",    0.5f },
 	{ "DCF2_RESO",      0.0f },
 	{ "DCF2_TYPE",      0.0f },
@@ -149,12 +155,17 @@ struct {
 	{ "DEL1_BPM",     180.0f },
 	{ "DEL1_BPMSYNC",   0.0f },
 	{ "DEL1_BPMHOST", 180.0f },
+	{ "REV1_WET",       0.0f },
+	{ "REV1_ROOM",      0.5f },
+	{ "REV1_DAMP",      0.5f },
+	{ "REV1_FEEDB",     0.5f },
+	{ "REV1_WIDTH",     0.0f },
 	{ "DYN1_COMPRESS",  0.0f },
-	{ "DYN1_LIMIT",     1.0f }
+	{ "DYN1_LIMITER",   1.0f }
 };
 
 
-QString synthv1_param::paramName ( synthv1::ParamIndex index )
+const char *synthv1_param::paramName ( synthv1::ParamIndex index )
 {
 	return synthv1_default_params[index].name;
 }
