@@ -1,7 +1,7 @@
 // synthv1_param.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,8 +24,6 @@
 
 #include "synthv1.h"
 
-#include <QString>
-
 
 //-------------------------------------------------------------------------
 // synthv1_param - decl.
@@ -34,7 +32,7 @@
 namespace synthv1_param
 {
 	// Default parameter name/value helpers.
-	QString paramName(synthv1::ParamIndex index);
+	const char *paramName(synthv1::ParamIndex index);
 	float paramDefaultValue(synthv1::ParamIndex index);
 };
 
