@@ -1,7 +1,7 @@
 // synthv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -117,6 +117,11 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	states << tr("Off");
 	states << tr("On");
 
+	m_ui.Dco1Bandl1Knob->insertItems(0, states);
+	m_ui.Dco1Bandl2Knob->insertItems(0, states);
+	m_ui.Dco2Bandl1Knob->insertItems(0, states);
+	m_ui.Dco2Bandl2Knob->insertItems(0, states);
+
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
 	m_ui.Dyn1LimiterKnob->insertItems(0, states);
 
@@ -126,8 +131,10 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Fla1WetKnob->setSpecialValueText(sOff);
 	m_ui.Pha1WetKnob->setSpecialValueText(sOff);
 	m_ui.Del1WetKnob->setSpecialValueText(sOff);
+	m_ui.Rev1WetKnob->setSpecialValueText(sOff);
 
-	m_ui.Del1BpmKnob->setSpecialValueText(tr("Auto"));
+	const QString& sAuto = tr("Auto");
+	m_ui.Del1BpmKnob->setSpecialValueText(sAuto);
 
 	// DCO octave limits.
 	m_ui.Dco1OctaveKnob->setMinimum(-4.0f);
@@ -216,18 +223,24 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Del1BpmKnob->setMinimum(3.6f);
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
 
+	// Reverb (stereo-)width limits.
+	m_ui.Rev1WidthKnob->setMinimum(-1.0f);
+	m_ui.Rev1WidthKnob->setMaximum(+1.0f);
 
 	// DCO1
 	setParamKnob(synthv1::DCO1_SHAPE1,  m_ui.Dco1Shape1Knob);
 	setParamKnob(synthv1::DCO1_WIDTH1,  m_ui.Dco1Width1Knob);
+	setParamKnob(synthv1::DCO1_BANDL1,  m_ui.Dco1Bandl1Knob);
 	setParamKnob(synthv1::DCO1_SHAPE2,  m_ui.Dco1Shape2Knob);
 	setParamKnob(synthv1::DCO1_WIDTH2,  m_ui.Dco1Width2Knob);
+	setParamKnob(synthv1::DCO1_BANDL2,  m_ui.Dco1Bandl2Knob);
 	setParamKnob(synthv1::DCO1_BALANCE, m_ui.Dco1BalanceKnob);
 	setParamKnob(synthv1::DCO1_DETUNE,  m_ui.Dco1DetuneKnob);
 	setParamKnob(synthv1::DCO1_PHASE,   m_ui.Dco1PhaseKnob);
 	setParamKnob(synthv1::DCO1_OCTAVE,  m_ui.Dco1OctaveKnob);
 	setParamKnob(synthv1::DCO1_TUNING,  m_ui.Dco1TuningKnob);
 	setParamKnob(synthv1::DCO1_GLIDE,   m_ui.Dco1GlideKnob);
+	setParamKnob(synthv1::DCO1_ENVTIME, m_ui.Dco1EnvTimeKnob);
 
 	QObject::connect(
 		m_ui.Dco1Shape1Knob, SIGNAL(valueChanged(float)),
@@ -423,14 +436,17 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	// DCO2
 	setParamKnob(synthv1::DCO2_SHAPE1,  m_ui.Dco2Shape1Knob);
 	setParamKnob(synthv1::DCO2_WIDTH1,  m_ui.Dco2Width1Knob);
+	setParamKnob(synthv1::DCO2_BANDL1,  m_ui.Dco2Bandl1Knob);
 	setParamKnob(synthv1::DCO2_SHAPE2,  m_ui.Dco2Shape2Knob);
 	setParamKnob(synthv1::DCO2_WIDTH2,  m_ui.Dco2Width2Knob);
+	setParamKnob(synthv1::DCO2_BANDL2,  m_ui.Dco2Bandl2Knob);
 	setParamKnob(synthv1::DCO2_BALANCE, m_ui.Dco2BalanceKnob);
 	setParamKnob(synthv1::DCO2_DETUNE,  m_ui.Dco2DetuneKnob);
 	setParamKnob(synthv1::DCO2_PHASE,   m_ui.Dco2PhaseKnob);
 	setParamKnob(synthv1::DCO2_OCTAVE,  m_ui.Dco2OctaveKnob);
 	setParamKnob(synthv1::DCO2_TUNING,  m_ui.Dco2TuningKnob);
 	setParamKnob(synthv1::DCO2_GLIDE,   m_ui.Dco2GlideKnob);
+	setParamKnob(synthv1::DCO2_ENVTIME, m_ui.Dco2EnvTimeKnob);
 
 	QObject::connect(
 		m_ui.Dco2Shape1Knob, SIGNAL(valueChanged(float)),
@@ -650,10 +666,18 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(valueChanged(float)),
 		SLOT(bpmSyncChanged()));
 
+	// Reverb
+	setParamKnob(synthv1::REV1_WET,   m_ui.Rev1WetKnob);
+	setParamKnob(synthv1::REV1_ROOM,  m_ui.Rev1RoomKnob);
+	setParamKnob(synthv1::REV1_DAMP,  m_ui.Rev1DampKnob);
+	setParamKnob(synthv1::REV1_FEEDB, m_ui.Rev1FeedbKnob);
+	setParamKnob(synthv1::REV1_WIDTH, m_ui.Rev1WidthKnob);
+
 	// Dynamics
 	setParamKnob(synthv1::DYN1_COMPRESS, m_ui.Dyn1CompressKnob);
 	setParamKnob(synthv1::DYN1_LIMITER,  m_ui.Dyn1LimiterKnob);
 
+
 	// Preset management
 	QObject::connect(m_ui.Preset,
 		SIGNAL(newPresetFile()),
@@ -701,6 +725,8 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 // Param kbob (widget) map accesors.
 void synthv1widget::setParamKnob ( synthv1::ParamIndex index, synthv1widget_knob *pKnob )
 {
+	pKnob->setDefaultValue(synthv1_param::paramDefaultValue(index));
+
 	m_paramKnobs.insert(index, pKnob);
 	m_knobParams.insert(pKnob, index);
 
@@ -716,13 +742,14 @@ synthv1widget_knob *synthv1widget::paramKnob ( synthv1::ParamIndex index ) const
 
 
 // Param port accessors.
-void synthv1widget::setParamValue ( synthv1::ParamIndex index, float fValue )
+void synthv1widget::setParamValue (
+	synthv1::ParamIndex index, float fValue, bool bDefault )
 {
 	++m_iUpdate;
 
 	synthv1widget_knob *pKnob = paramKnob(index);
 	if (pKnob)
-		pKnob->setValue(fValue);
+		pKnob->setValue(fValue, bDefault);
 
 	updateParamEx(index, fValue);
 
@@ -859,8 +886,8 @@ void synthv1widget::resetParamValues (void)
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		synthv1::ParamIndex index = synthv1::ParamIndex(i);
-		float fValue = synthv1_param::paramDefaultValue(index);
-		setParamValue(index, fValue);
+		const float fValue = synthv1_param::paramDefaultValue(index);
+		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 		m_params_ab[index] = fValue;
 	}
@@ -930,8 +957,8 @@ void synthv1widget::loadPreset ( const QString& sFilename )
 		}
 	}
 
-	resetParamValues();
 	resetParamKnobs();
+	resetParamValues();
 
 	pSynth->reset();
 
@@ -961,14 +988,17 @@ void synthv1widget::loadPreset ( const QString& sFilename )
 							synthv1::ParamIndex index = synthv1::ParamIndex(
 								eParam.attribute("index").toULong());
 							const QString& sName = eParam.attribute("name");
-							if (!sName.isEmpty() && s_hash.contains(sName))
+							if (!sName.isEmpty()) {
+								if (!s_hash.contains(sName))
+									continue;
 								index = s_hash.value(sName);
+							}
 							float fValue = eParam.text().toFloat();
 						//--legacy support < 0.3.0.4 -- begin
 							if (index == synthv1::DEL1_BPM && fValue < 3.6f)
 								fValue *= 100.0f;
 						//--legacy support < 0.3.0.4 -- end.
-							setParamValue(index, fValue);
+							setParamValue(index, fValue, true);
 							updateParam(index, fValue);
 							m_params_ab[index] = fValue;
 						}
