@@ -1,7 +1,7 @@
 // synthv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -45,7 +45,8 @@ public:
 	synthv1widget(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
 
 	// Param port accessors.
-	void setParamValue(synthv1::ParamIndex index, float fValue);
+	void setParamValue(
+		synthv1::ParamIndex index, float fValue, bool bDefault = false);
 	float paramValue(synthv1::ParamIndex index) const;
 
 	// Param kbob (widget) mapper.
